"""
Autodocuments this library.
"""
import os
import shutil
from pathlib import Path


EXAMPLEROOT = Path(__file__).parent.parent.joinpath('examples', 'learn')


def add_parser_arguments(parser):
    parser.add_argument('--source', default='xor_clf', help='source example name')
    parser.add_argument('--name', default='myproj', help='project name')


def main(args):
    dst = Path(os.getcwd()).joinpath(args.name)
    print('making directory {}'.format(dst))
    shutil.copytree(src=EXAMPLEROOT.joinpath(args.source), dst=dst)
    if dst.joinpath('__init__.py').is_file():
        os.remove(dst.joinpath('__init__.py'))
    if dst.joinpath('saves').is_dir():
        shutil.rmtree(dst.joinpath('saves'))



import unittest
from wrangl.examples.preprocess import repeat_string as T


class TestRepeatString(unittest.TestCase):
    expect = [s * 10 for s in T.MyDataset.strings]

    def test_no_pool_loader(self):
        out = T.run_no_process()
        self.assertListEqual(self.expect, out)

    def test_ordered(self):
        out = T.run_ordered()
        self.assertListEqual(self.expect, out)

    def test_repeat_unordered(self):
        out = T.run_unordered()
        self.assertSetEqual(set(self.expect), set(out))



#!/usr/bin/env python
import ray
import json
import hydra
import logging
from wrangl.learn import SupervisedModel
from wrangl.data import IterableDataset, Processor


logger = logging.getLogger(__name__)


@ray.remote
class MyProcessor(Processor):

    def process(self, row):
        return dict(sent=row['sentence'], label_text=row['gold_label'], label_idx=['positive', 'neutral', 'negative'].index(row['gold_label']))


def load_data(fname):
    data = []
    with open(fname) as f:
        for line in f:
            row = json.loads(line)
            if row['gold_label'] not in {None, 'mixed'}:
                data.append(row)
    return data


@hydra.main(config_path='conf', config_name='default')
def main(cfg):
    Model = SupervisedModel.load_model_class(cfg.model)
    pool = ray.util.ActorPool([MyProcessor.remote() for _ in range(cfg.num_workers)])
    train = IterableDataset(load_data(cfg.ftrain), pool=pool, shuffle=True)
    val = IterableDataset(load_data(cfg.feval), pool=pool)
    Model.run_train_test(cfg, train, val)


if __name__ == '__main__':
    main()



import os
import importlib
import lightning as L
from torch import optim
from hydra.utils import get_original_cwd


class BaseModel(L.LightningModule):

    @classmethod
    def load_model_class(cls, model_name, model_dir='model', root_dir=None):
        """
        Loads a model from file. Note that model class must be `Model`.

        Args:
            model_name: name of model to load.
            model_dir: directory of model files.
            root_dir: source directory. If not specified, then it is inferred from HydraConfig's launch directory.

        Suppose we have our model files in a directory `mymodels`, and in `mymodels/cool_model.py` we have:

        ```
        from wrangl.learn import SupervisedModel
        class Model(SupervisedModel):
            ...
        ```

        To load this model we would call in `train.py`:
        ```python
        MyModel = SupervisedModel.load_model_class('cool_model', 'mymodels')
        ```
        """
        if root_dir is None:
            root_dir = get_original_cwd()
        fname = os.path.join(root_dir, model_dir, '{}.py'.format(model_name))
        spec = importlib.util.spec_from_file_location(model_name, fname)
        module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(module)
        Model = module.Model
        return Model

    def __init__(self, cfg):
        """
        Instiate a model from config file.

        Args:
            cfg: `omegaconf.OmegaConf` config to use.

        By default, this config is created by [Hydra](https://hydra.cc) from your config files.
        If you'd like to use a `dict`, instead:

        ```python
        conf = OmegaConf.create({"var1" : "v", "var2" : [1, {"a": "1", "b": "2", 3: "c"}]})
        model = Model(conf)
        ```

        For more info on OmegaConf, see [their docs](https://omegaconf.readthedocs.io/).
        """
        super().__init__()
        self.save_hyperparameters(cfg)
        self.pred_samples = []

    def configure_optimizers(self):
        """
        Returns a the `torch.optim.Optimizer` to use for this model.
        """
        opt = self.hparams.optim
        Optimizer = getattr(optim, opt.name)
        return Optimizer(self.parameters(), **opt.params)



"""
Boilerplate that combines Hydra conf and PytorchLightning for supervised training.
"""
import os
import json
import torch
import random
import logging
import pathlib
import lightning as L
from typing import List
from omegaconf import OmegaConf
from torch.utils.data import DataLoader
from .callbacks import WandbTableCallback, GitCallback
from .metrics import Accuracy
from .model import BaseModel
from lightning.pytorch import callbacks as C
from lightning.pytorch.loggers import WandbLogger, CSVLogger


class SupervisedModel(BaseModel):
    """
    Supervised learning Pytorch lightning module.
    You should overload this model as you see fit.
    """

    def get_callbacks(self):
        """
        Returns a list of `pytorch_lightning.callbacks.Callback`s to use for training.
        """
        callbacks = []
        if self.hparams.git.enable:
            callbacks.append(GitCallback())
        if self.hparams.wandb.enable:
            callbacks.append(WandbTableCallback())
        return callbacks

    def featurize(self, batch: list):
        """
        Converts a batch of examples to features.
        By default this returns the batch as is.

        Alternatively you may want to set `collate_fn: "ignore"` in your config and use `featurize` to convert raw examples into features.
        """
        return batch

    def compute_metrics(self, pred: list, gold: list, batch: list) -> dict:
        """
        Computes metrics between predictions and ground truths.
        """
        m = Accuracy()
        return m(pred, gold)

    def compute_loss(self, out, feat, batch) -> torch.Tensor:
        """
        Computes loss from inputs and model output.
        You must implement this function.

        Args:
            out: model output from `wrangl.learn.supervised.Model.forward`.
            feat: featurize output from `wrangl.learn.supervised.Model.featurize`.
            batch: Dataset iterator output.

        """
        raise NotImplementedError()

    def extract_context(self, out, feat, batch) -> List:
        """
        Extract example context from inputs and model output.
        You must implement this function.

        Args:
            out: model output from `wrangl.learn.supervised.Model.forward`.
            feat: featurize output from `wrangl.learn.supervised.Model.featurize`.
            batch: Dataset iterator output.

        """

        raise NotImplementedError()

    def extract_pred(self, out, feat, batch) -> List:
        """
        Extract model predictions from inputs and model output.
        You must implement this function.

        Args:
            out: model output from `wrangl.learn.supervised.Model.forward`.
            feat: featurize output from `wrangl.learn.supervised.Model.featurize`.
            batch: Dataset iterator output.
        """

        raise NotImplementedError()

    def extract_gold(self, feat, batch) -> List:
        """
        Extract ground truths from inputs and model output.
        You must implement this function.

        Args:
            out: model output from `wrangl.learn.supervised.Model.forward`.
            feat: featurize output from `wrangl.learn.supervised.Model.featurize`.
            batch: Dataset iterator output.

        """

        raise NotImplementedError()

    def forward(self, feat, batch):
        """
        Computes model output from inputs.
        You must implement this function.

        Args:
            feat: featurize output from `wrangl.learn.supervised.Model.featurize`.
            batch: Dataset iterator output.

        """
        raise NotImplementedError()

    def infer(self, feat, batch):
        """
        Computes model output from inputs for inference.
        Defaults to `wrangl.learn.supervised.Model.forward`.

        Args:
            feat: featurize output from `wrangl.learn.supervised.Model.featurize`.
            batch: Dataset iterator output.

        """
        return self.forward(feat, batch)

    def infer_batch_size(self, batch):
        """
        Infers batch size from batch of raw examples.
        """
        if isinstance(batch, list):
            return len(batch)
        if isinstance(batch, dict):
            first_key = list(batch.keys())[0]
            return len(batch[first_key])
        return None

    def training_step(self, batch, batch_id):
        """
        One training step in PytorchLightning.
        You should not have to modify this.
        For more information, see `pytorch_lightning.training_step`.
        """
        feat = self.featurize(batch)
        out = self.forward(feat, batch)
        loss = self.compute_loss(out, feat, batch)
        self.log('loss', loss, batch_size=self.infer_batch_size(batch))
        perplexity = torch.exp(loss)
        self.log('ppl', perplexity, batch_size=self.infer_batch_size(batch))
        return loss

    def test_step(self, batch, batch_id, split='test'):
        """
        One test step in PytorchLightning.
        You should not have to modify this.
        For more information, see `pytorch_lightning.validation_step`.
        """
        return self.validation_step(batch, batch_id, split='test')

    def predict_step(self, batch, batch_id):

        """
        One prediction step in PytorchLightning.
        You should not have to modify this.
        For more information, see `pytorch_lightning.prediction_step`.
        """
        feat = self.featurize(batch)
        out = self.infer(feat, batch)
        pred = self.extract_pred(out, feat, batch)
        return pred

    def validation_step(self, batch, batch_id, split='val'):
        """
        One validation step in PytorchLightning.
        You should not have to modify this.
        For more information, see `pytorch_lightning.validation_step`.

        A core difference here is that Wrangl will
        1. compute metrics automatically
        2. extract example contexts, predictions, and ground truths
        3. save predictions to disk (and upload to Wandb etc.) for debugging
        """
        feat = self.featurize(batch)
        context = self.extract_context(feat, batch)
        gold = self.extract_gold(feat, batch)

        out = self.infer(feat, batch)
        pred = self.extract_pred(out, feat, batch)

        metrics = self.compute_metrics(pred, gold, batch)
        for k, v in metrics.items():
            self.log('{}_{}'.format(split, k), v, batch_size=self.infer_batch_size(batch))

        # for logging to wandb
        sample_ids = list(range(len(context)))
        random.shuffle(sample_ids)
        sample_ids = sample_ids[:self.hparams.val_sample_size]

        random.shuffle(self.pred_samples)
        for i in sample_ids:
            self.pred_samples.insert(0, (context[i], pred[i], gold[i]))
        self.pred_samples = self.pred_samples[:self.hparams.val_sample_size]

        with open('pred_samples.json', 'wt') as f:
            data = [dict(context=repr(context), gen=repr(gen), gold=repr(gold)) for context, gen, gold in self.pred_samples]
            json.dump(data, f, indent=2)

    @classmethod
    def run_train_test(cls, cfg: OmegaConf, train_dataset: torch.utils.data.Dataset, eval_dataset: torch.utils.data.Dataset, model_kwargs=None):
        """
        Creates a model according to config and trains/evaluates it.

        Args:
            cfg: config to use.
            train_dataset: dataset for training.
            eval_dataset: dataset for validation.
            model_kwargs: key ward arguments for model constructor.

        This will additionally set up according to your config file
            - manual seeds
            - early stopping
            - checkpoint saving
            - gradient clipping
            - callbacks (including an always on CSV logger)
        """
        model_kwargs = model_kwargs or {}
        L.seed_everything(seed=cfg.seed, workers=True)
        dout = os.getcwd()

        logger = logging.getLogger(name='{}:train_test'.format(cls.__name__))
        logger.info('Logging to {}'.format(dout))

        checkpoint = C.ModelCheckpoint(
            dirpath=dout,
            monitor=cfg.early_stopping.monitor,
            mode=cfg.early_stopping.mode,
            filename='{epoch:02d}-{' + cfg.early_stopping.monitor + ':.4f}',
            every_n_epochs=1,
            save_top_k=3,
            save_last=True,
            verbose=True,
        )

        train_logger = [CSVLogger(save_dir=dout, name='logs', flush_logs_every_n_steps=cfg.flush_logs_every_n_steps)]
        if cfg.wandb.enable:
            train_logger.append(WandbLogger(project=cfg.wandb.project, name=cfg.wandb.name, entity=cfg.wandb.entity, save_dir=cfg.wandb.dir))

        fconfig = os.path.join(dout, 'config.yaml')
        OmegaConf.save(config=cfg, f=fconfig)

        model = cls(cfg, **model_kwargs)

        logger.info('Loading data')
        if cfg.collate_fn == 'auto':
            collate_fn = None
        elif cfg.collate_fn == 'ignore':
            def collate_fn(batch):
                return batch
        else:
            collate_fn = cls.collate_fn
        train_loader = DataLoader(train_dataset, batch_size=cfg.batch_size, collate_fn=collate_fn)
        eval_loader = DataLoader(eval_dataset, batch_size=cfg.val_batch_size, collate_fn=collate_fn)

        trainer = L.Trainer(
            precision=cfg.precision,
            strategy=cfg.strategy,
            devices=cfg.devices,
            benchmark=True,
            default_root_dir=dout,
            gradient_clip_val=cfg.grad_clip_norm,
            gradient_clip_algorithm='norm',
            log_every_n_steps=cfg.log_every_n_steps,
            val_check_interval=1 if cfg.debug else cfg.val_check_interval,
            limit_val_batches=cfg.limit_val_batches,
            max_steps=cfg.max_steps,
            logger=train_logger,
            callbacks=[checkpoint] + model.get_callbacks(),
        )
        if not cfg.test_only:
            ckpt_path = None
            if cfg.autoresume:
                ckpt_path = os.path.join(dout, 'last.ckpt')
                if not os.path.isfile(ckpt_path):
                    ckpt_path = None
            trainer.fit(model, train_loader, eval_loader, ckpt_path=ckpt_path)

        model = cls.load_from_checkpoint(os.path.join(dout, 'last.ckpt'))
        result = trainer.test(model, eval_loader, verbose=True, test=False)
        with open(os.path.join(dout, 'eval_pred.json'), 'wt') as f:
            json.dump(result, f, indent=2)
        result = trainer.test(model, eval_loader, verbose=True, test=True)
        with open(os.path.join(dout, 'eval_test.json'), 'wt') as f:
            json.dump(result, f, indent=2)
        logger.info('Finished!')

    @classmethod
    def run_inference(cls, cfg: OmegaConf, fcheckpoint: str, eval_dataset: torch.utils.data.DataLoader, model_kwargs=None, test=False):
        """
        Loads a model according to config and runs inference.

        Args:
            cfg: config to use.
            fcheckpoint: model checkpoint to load.
            eval_dataset: dataset for validation.
            model_kwargs: key ward arguments for model constructor.
            test: whether to evaluate metrics

        Returns:
            if `test==True` then returns a dictionary of scores, otherwise returns a list of model predictions.
        """
        model_kwargs = model_kwargs or {}
        model = cls.load_from_checkpoint(fcheckpoint, **model_kwargs)

        if cfg.collate_fn == 'auto':
            collate_fn = None
        elif cfg.collate_fn == 'ignore':
            def collate_fn(batch):
                return batch
        else:
            collate_fn = cls.collate_fn
        eval_loader = DataLoader(eval_dataset, batch_size=cfg.batch_size, collate_fn=collate_fn)

        trainer = L.Trainer(
            precision=cfg.precision,
            strategy=cfg.strategy,
            devices=cfg.devices,
            benchmark=True,
        )
        if test:
            result = trainer.test(model, eval_loader, verbose=True)
        else:
            result = []
            for b in trainer.predict(model, eval_loader):
                result.extend(b)
        return result

    @classmethod
    def load_experiment(cls, dexp):
        """
        Loads experiment from disk.
        """
        import pandas as pd
        dexp = pathlib.Path(dexp)
        config = OmegaConf.to_container(OmegaConf.load(dexp.joinpath('config.yaml')), resolve=True)

        # make sure collection exists for project
        df = []
        for flog in dexp.joinpath('logs').glob('*/*.csv'):
            df.append(pd.read_csv(flog))
        if df:
            logs = pd.concat(df).to_dict()
        else:
            logs = {}
        return config, logs



#!/usr/bin/env python
import hydra
import torch
import random
import logging
from wrangl.learn import SupervisedModel
from torch.utils.data import Dataset


logger = logging.getLogger(__name__)


class MyDataset(list, Dataset):
    pass


def generate_dataset(n, seed=0):
    rng = random.Random(seed)
    dataset = MyDataset()
    for _ in range(n):
        raw = dict(x=rng.uniform(-1, 1), y=rng.uniform(-1, 1))
        raw.update(dict(
            feat=torch.tensor([raw['x'], raw['y']], dtype=torch.float32),
            label=1 if raw['x'] > 0 and raw['y'] > 0 else 0,
        ))
        dataset.append(raw)
    return dataset


@hydra.main(config_path='conf', config_name='default')
def main(cfg):
    Model = SupervisedModel.load_model_class(cfg.model)
    train = generate_dataset(10000)
    val = generate_dataset(1000)
    Model.run_train_test(cfg, train, val)


if __name__ == '__main__':
    main()



"""
Autodocuments this library.
"""
import os
import shutil
from pdoc import pdoc, render
from pathlib import Path
from datetime import date


DOCDIR = Path(__file__).parent.parent.parent.joinpath('docs')


def add_parser_arguments(parser):
    parser.add_argument('--host', default=os.environ.get('WRANGL_DOCS_HOST'), help='where the docs will be hosted')
    parser.add_argument('--format', default='google', help='doc format')
    parser.add_argument('--logo', help='logo image')
    parser.add_argument('--logo_link', help='logo link')


def main(args):
    render.configure(
        docformat=args.format,
        edit_url_map=dict(wrangl=args.host) if args.host else None,
        footer_text='Copyright @{} by Victor Zhong'.format(date.today().year),
        logo=args.logo,
        logo_link=args.logo_link,
        math=True,
        search=True,
        show_source=True,
        template_directory=DOCDIR.joinpath('templates'),
    )
    pdoc('wrangl', output_directory=DOCDIR.joinpath('build'))
    os.remove(DOCDIR.joinpath('build', 'index.html'))
    # use the package html as the index
    shutil.copy(DOCDIR.joinpath('build', 'wrangl.html'), DOCDIR.joinpath('build', 'index.html'))



"""
Wrangl implements two learning boilerplates:

- `wrangl.learn.supervised` for Supervised Learning, mostly using PytorchLightning.

.. include:: ../examples/learn/README.md
"""
from .supervised import SupervisedModel



# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
from hydra.core.config_search_path import ConfigSearchPath
from hydra.plugins.search_path_plugin import SearchPathPlugin


class WranglSearchPathPlugin(SearchPathPlugin):

    def manipulate_search_path(self, search_path: ConfigSearchPath) -> None:
        # Appends the search path for this plugin to the end of the search path
        # Note that foobar/conf is outside of the example plugin module.
        # There is no requirement for it to be packaged with the plugin, it just needs
        # be available in a package.
        # Remember to verify the config is packaged properly (build sdist and look inside,
        # and verify MANIFEST.in is correct).
        search_path.append(
            provider="wrangl-searchpath-plugin", path="pkg://wrangl/conf"
        )



from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
from . import plotter, docs, projects
import unittest


def main():
    parser = ArgumentParser(formatter_class=ArgumentDefaultsHelpFormatter)
    subs = parser.add_subparsers(title='command', dest='cmd')

    plotter.add_parser_arguments(subs.add_parser('plot'))
    docs.add_parser_arguments(subs.add_parser('autodoc'))
    projects.add_parser_arguments(subs.add_parser('project'))

    autotest = subs.add_parser('autotest')
    autotest.add_argument('--dtest', default='tests', help='test directory')

    args = parser.parse_args()

    if args.cmd == 'plot':
        plotter.main(args)
    elif args.cmd == 'project':
        projects.main(args)
    elif args.cmd == 'autodoc':
        docs.main(args)
    elif args.cmd == 'autotest':
        tests = unittest.TestLoader().discover(args.dtest)
        unittest.runner.TextTestRunner().run(tests)


if __name__ == '__main__':
    main()



from collections import defaultdict


class Metric:
    """
    Interface for a metric.
    """

    def compute_one(self, pred, gold):
        """
        Computes metrics for one example.
        You must implement this.

        Args:
            pred: single prediction.
            gold: corresponding ground truth.
        """
        raise NotImplementedError()

    def __call__(self, pred, gold):
        return self.forward(pred, gold)

    def forward(self, pred: list, gold: list) -> dict:
        """
        Computes metric over list of predictions and ground truths and returns a dictionary of scores.

        Args:
            pred: list of predictions.
            gold: corresponding ground truths.
        """
        metrics = defaultdict(list)
        for pi, gi in zip(pred, gold):
            m = self.compute_one(pi, gi)
            for k, v in m.items():
                metrics[k].append(v)
        return {k: sum(v)/len(v) for k, v in metrics.items()}


class Accuracy(Metric):
    """
    Computes exact match accuracy under the key "acc".
    """

    def compute_one(self, pred, gold):
        return {'acc': pred == gold}


class MSE(Metric):
    """
    Computes mean squared error under the key "mse".
    """

    def compute_one(self, pred, gold):
        return {'mse': (pred - gold) ** 2}


class SetF1(Metric):
    """
    Computes F1 score under the key "f1", "precision", and "recall".
    Here, both single prediction and ground truth are assumed to be a `set`.
    """

    def compute_one(self, pred: set, gold: set):
        common = pred.intersection(gold)
        precision = len(common) / max(1, len(pred))
        recall = len(common) / max(1, len(gold))
        denom = precision + recall
        f1 = (precision * recall * 2 / denom) if denom > 0 else 0
        return dict(f1=f1, precision=precision, recall=recall)

    def __call__(self, pred, gold, ignore_empty=False):
        metrics = dict(f1=[], recall=[], precision=[])
        for pi, gi in zip(pred, gold):
            if ignore_empty and (not gi or gi == {''}):
                continue
            for k, v in self.compute_one(pi, gi).items():
                metrics[k].append(v)
        return {k: (sum(v)/len(v)) for k, v in metrics.items()}


class Rouge(Metric):
    """
    Computes Rouge score under the key "rouge".
    Here, both single prediction and ground truth are assumed to be a `str`.
    You must install `rouge_scorer` for this to work.
    """

    def __init__(self):
        from rouge_score import rouge_scorer
        self.scorer = rouge_scorer.RougeScorer(['rougeL'], use_stemmer=True)

    def compute_one(self, pred: str, gold: str):
        return dict(rouge=self.scorer.score(pred, gold)['rougeL'].fmeasure)



"""
Callbacks that can be used during PytorchLightning training.
"""

import json
import wandb
from hydra.utils import get_original_cwd
from lightning.pytorch.callbacks import Callback


class WandbTableCallback(Callback):
    """
    Uploads sample predictions to Wandb.
    """

    def on_validation_end(self, trainer, model):
        wandb_logger = [exp for exp in trainer.logger.experiment if isinstance(exp, wandb.sdk.wandb_run.Run)]
        assert wandb_logger
        run = wandb.Api().run(path=wandb_logger[0].path)
        for artifact in run.logged_artifacts():
            artifact.delete(delete_aliases=True)
        table = wandb.Table(columns=['context', 'pred', 'gold'])
        for context, gen, gold in model.pred_samples:
            table.add_data(repr(context), repr(gen), repr(gold))
        wandb.log(dict(gen=table))


class GitCallback(Callback):
    """
    Dumps git diffs to work directory.
    """

    def on_init_end(self, trainer=None):
        import git
        repo = git.Repo(get_original_cwd(), search_parent_directories=True)
        with open('git.patch.diff', 'wt') as f:
            f.write(repo.git.diff(repo.head.commit.tree))
        with open('git.head.json', 'wt') as f:
            c = repo.head.commit
            json.dump(dict(hexsha=c.hexsha, message=c.message, date=c.committed_date, committer=c.committer.name), f, indent=2)



import unittest
from wrangl.learn import metrics as M


class TestMetrics(unittest.TestCase):

    def test_accuracy(self):
        m = M.Accuracy()
        self.assertDictEqual(dict(acc=True), m.compute_one('foo', 'foo'))
        self.assertDictEqual(dict(acc=False), m.compute_one('bar', 'foo'))
        self.assertDictEqual(dict(acc=0.5), m(['foo', 'foo'], ['foo', 'bar']))

    def test_f1(self):
        m = M.SetF1()
        ax = {2, 4, 5, 6}, {1, 2}
        bx = {1, 4}, {1, 2, 4, 5}
        ay = dict(precision=1/4, recall=1/2, f1=2*0.25*0.5/(0.25+0.5))
        by = dict(precision=2/2, recall=2/4, f1=2*1*0.5/(1+0.5))
        self.assertDictEqual(ay, m.compute_one(*ax))
        self.assertDictEqual(by, m.compute_one(*bx))
        self.assertDictEqual({k: (ay[k]+by[k])/2 for k in ay.keys()}, m([ax[0], bx[0]], [ax[1], bx[1]]))



import unittest
from wrangl.examples.preprocess import jsonl_files as T


class TestLoadJsonlFiles(unittest.TestCase):

    def test_fileloader(self):
        data, output = T.load_files()
        self.assertListEqual(data, output)


if __name__ == '__main__':
    unittest.main()



import unittest
from wrangl.examples.preprocess import sql_db as T


class TestLoadSQLDB(unittest.TestCase):

    def test_dbloader(self):
        data, output = T.load_db()
        self.assertListEqual(data, output)


if __name__ == '__main__':
    unittest.main()



import os
import csv
import typing
import pathlib
import logging
import plotille
import pandas as pd
import ujson as json


def add_parser_arguments(parser):
    parser.add_argument('dlogs', nargs='+', help='log directory')
    parser.add_argument('--type', choices=('rl', 'supervised', 'auto'), default='auto', help='type of experiment')
    parser.add_argument('--curves', choices=('train', 'eval', 'both'), default='eval', help='type of curve to plot')
    parser.add_argument('--window', help='smoothing window', type=int, default=1)
    parser.add_argument('--width', help='plot width', type=int, default=60)
    parser.add_argument('--height', help='plot height', type=int, default=10)
    parser.add_argument('-n', '--path_parts', help='how many last folders to use as name', type=int, default=2)
    parser.add_argument('-x', help='x axis', default='auto')
    parser.add_argument('-y', help='y axis', default='auto')


def detect_log_type(dlogs):
    dlog = dlogs[0]
    if dlog.joinpath('metrics.log.json').exists():
        return 'supervised'
    elif dlog.joinpath('metrics.log.jsonl').exists():
        return 'rl'
    else:
        return NotImplementedError('Unknown log type in {}'.format(dlog))


def load_supervised(dlogs: typing.List[pathlib.Path], n: int = 2):
    ret = []
    for dlog in dlogs:
        for root, dirs, files in os.walk(dlog):
            if 'metrics.csv' in files:
                fmetric = os.path.join(root, 'metrics.csv')
                log = []
                with open(fmetric, 'rt') as f:
                    reader = csv.reader(f)
                    header = next(reader)
                    key = '/'.join(root.split('/')[-n:])
                    for r in reader:
                        d = dict(zip(header, r))
                        d['train_steps'] = d['step']
                        del d['step']
                        for k, v in d.items():
                            x = pd.to_numeric(v, errors='ignore')
                            if not pd.isna(x):
                                d[k] = x
                        train = {k: v for k, v in d.items() if not k.startswith('val')}
                        train['type'] = 'train'
                        val = d.copy()
                        val['type'] = 'eval'
                        log.append(train)
                        log.append(val)
                ret.append((key, log))
    return ret


def load_rl(dlogs: typing.List[pathlib.Path], n: int = 2):
    ret = []
    for dlog in dlogs:
        name = '/'.join(dlog.parts[-n:])
        log = []
        flog = dlog.joinpath('metrics.log.jsonl')
        if not flog.exists():
            logging.error('Could not find {}'.format(flog))
            continue
        with flog.open('rt') as f:
            for i, line in enumerate(f):
                try:
                    log.append(json.loads(line))
                except Exception as e:
                    print('Could not load line {}: {}'.format(i+1, line))
                    raise e
        ret.append((name, log))
    return ret


def main(args):
    args.dlogs = [pathlib.Path(d) for d in args.dlogs]
    if args.type == 'auto':
        args.type = detect_log_type(args.dlogs)
    if args.x == 'auto':
        args.x = 'train_steps'
    if args.type == 'supervised':
        logs = load_supervised(args.dlogs, n=args.path_parts)
        if args.y == 'auto':
            args.y = 'loss'
    elif args.type == 'rl':
        logs = load_rl(args.dlogs, n=args.path_parts)
        if args.y == 'auto':
            args.y = 'mean_episode_return'
    else:
        raise NotImplementedError()

    fig = plotille.Figure()
    kwargs = {}
    for exp, log in logs:
        if log:
            if args.curves in {'train', 'both'}:
                df = pd.DataFrame([e for e in log if e['type'] == 'train' and e[args.x] not in {'', None} and e.get(args.y) not in {'', None}])
                fig.plot(
                    X=df[args.x],
                    Y=df[args.y].rolling(args.window, min_periods=1).mean(),
                    label='train {}'.format(exp),
                    **kwargs
                )
            if args.curves in {'eval', 'both'}:
                df = pd.DataFrame([e for e in log if e['type'] == 'eval' and e[args.x] not in {'', None} and e.get(args.y) not in {'', None}])
                fig.plot(
                    X=df[args.x],
                    Y=df[args.y].rolling(args.window, min_periods=1).mean(),
                    label='eval {}'.format(exp),
                    **kwargs
                )
    fig.x_label = args.x
    fig.y_label = args.y
    fig.height = args.height
    fig.width = args.width

    print(fig.show(legend=True))
